/*  Copyright (C) 2012 Gerald Mwangi

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file VirtualSlotTestReader.cpp
 *  Implementation of parameter class VirtualSlotTestReader.
 *  \author <a href="mailto:gerald.mwangi@gmx.de">
 *      Gerald Mwangi</a>
 *  \date 28.11.2012
 */

#include <charon-core/ParameteredObject.hxx>
#include "VirtualSlotTestReader.h"

VirtualSlotTestReader::VirtualSlotTestReader(const std::string& name) :
		ParameteredObject(
			"VirtualSlotTestReader", name,
			"<h2>Tests if the data passed to its input matches the data "
			"generated by VirtualSlotTestWriter</h2><br>"
			"Tests if the data passed to its input matches the data "
			"generated by VirtualSlotTestWriter"
		)
{

	ParameteredObject::_addInputSlot(
		integer, "integer",
		"integer",
		"int");
	ParameteredObject::_addInputSlot(
		image, "image",
		"image",
		"string");

}

void VirtualSlotTestReader::execute() {
	//Test if the answer is 42
	if(integer!=42)
		raise("The answer is not 42!");
	//Test image
	if (image()!="image") {
		raise("Image not correctly passed!");
	}

	//Test image again, with data retrieval as applied for multislots
	std::set<AbstractSlot<std::string>*>::const_iterator it=image.begin();
	Slot* sl=(Slot*)(*it);
	OutputSlotIntf* out=dynamic_cast<OutputSlotIntf*>(sl);
	if(!out) {
		raise("cast of image failed! ");
	}
	const OutputSlot<std::string>* temp =
		dynamic_cast<const OutputSlot<std::string>*>(out->getDataSlot());

	std::string im2=temp->operator()();
	if(im2 != "image") {
		raise("Image has incorrect dimensions!");
	}
}

// the following functions are needed
// for class VirtualSlotTestReader to work as a charon plugin.
extern "C" virtualslottestreader_DECLDIR ParameteredObject*
		create(const std::string& name, ParameteredObject::template_type) {
	return new VirtualSlotTestReader(name);
}

extern "C" virtualslottestreader_DECLDIR void destroy(ParameteredObject* b) {
	delete b;
}

/// Report build configuration to prevent linking of incompatibel runtime libs
extern "C" virtualslottestreader_DECLDIR ParameteredObject::build_type getBuildType() {
#ifdef _DEBUG
	return ParameteredObject::DEBUG_BUILD;
#else
	return ParameteredObject::RELEASE_BUILD;
#endif
}
